@@ -1,6 +1,6 @@
 # frozen_string_literal: true
 
-require 'rack/request'
+require_relative '../../request'
 
 module Rack
   module Auth
@@ -27,7 +27,7 @@ module Rack
       end
 
       def scheme
-        @scheme ||= parts.first && parts.first.downcase
+        @scheme ||= parts.first&.downcase
       end
 
       def params
