@@ -2,6 +2,6 @@
 
 class AnUnderscoreApp
   def self.call(env)
-    [200, { 'Content-Type' => 'text/plain' }, ['OK']]
+    [200, { 'content-type' => 'text/plain' }, ['OK']]
   end
 end
