@@ -2,6 +2,12 @@
 
 require_relative 'helper'
 
+separate_testing do
+  require_relative '../lib/rack/auth/basic'
+  require_relative '../lib/rack/mock_request'
+  require_relative '../lib/rack/lint'
+end
+
 describe Rack::Auth::Basic do
   def realm
     'WallysWorld'
@@ -9,7 +15,7 @@ describe Rack::Auth::Basic do
 
   def unprotected_app
     Rack::Lint.new lambda { |env|
-      [ 200, { 'Content-Type' => 'text/plain' }, ["Hi #{env['REMOTE_USER']}"] ]
+      [ 200, { 'content-type' => 'text/plain' }, ["Hi #{env['REMOTE_USER']}"] ]
     }
   end
 
@@ -34,8 +40,8 @@ describe Rack::Auth::Basic do
   def assert_basic_auth_challenge(response)
     response.must_be :client_error?
     response.status.must_equal 401
-    response.must_include 'WWW-Authenticate'
-    response.headers['WWW-Authenticate'].must_match(/Basic realm="#{Regexp.escape(realm)}"/)
+    response.must_include 'www-authenticate'
+    response.headers['www-authenticate'].must_match(/Basic realm="#{Regexp.escape(realm)}"/)
     response.body.must_be :empty?
   end
 
@@ -62,7 +68,7 @@ describe Rack::Auth::Basic do
     request 'HTTP_AUTHORIZATION' => 'Digest params' do |response|
       response.must_be :client_error?
       response.status.must_equal 400
-      response.wont_include 'WWW-Authenticate'
+      response.wont_include 'www-authenticate'
     end
   end
 
@@ -70,7 +76,7 @@ describe Rack::Auth::Basic do
     request 'HTTP_AUTHORIZATION' => '' do |response|
       response.must_be :client_error?
       response.status.must_equal 400
-      response.wont_include 'WWW-Authenticate'
+      response.wont_include 'www-authenticate'
     end
   end
 
@@ -86,7 +92,7 @@ describe Rack::Auth::Basic do
     request 'HTTP_AUTHORIZATION' => auth do |response|
       response.must_be :client_error?
       response.status.must_equal 400
-      response.wont_include 'WWW-Authenticate'
+      response.wont_include 'www-authenticate'
     end
   end
 
