@@ -2,6 +2,13 @@
 
 require_relative 'helper'
 
+separate_testing do
+  require_relative '../lib/rack/config'
+  require_relative '../lib/rack/lint'
+  require_relative '../lib/rack/builder'
+  require_relative '../lib/rack/mock_request'
+end
+
 describe Rack::Config do
   it "accept a block that modifies the environment" do
     app = Rack::Builder.new do
@@ -10,7 +17,7 @@ describe Rack::Config do
         env['greeting'] = 'hello'
       end
       run lambda { |env|
-        [200, { 'Content-Type' => 'text/plain' }, [env['greeting'] || '']]
+        [200, { 'content-type' => 'text/plain' }, [env['greeting'] || '']]
       }
     end
 
