@@ -1,10 +1,12 @@
 # frozen_string_literal: true
 
-require 'minitest/global_expectations/autorun'
-require 'stringio'
-require 'rack/lint'
-require 'rack/logger'
-require 'rack/mock'
+require_relative 'helper'
+
+separate_testing do
+  require_relative '../lib/rack/logger'
+  require_relative '../lib/rack/lint'
+  require_relative '../lib/rack/mock_request'
+end
 
 describe Rack::Logger do
   app = lambda { |env|
@@ -13,7 +15,7 @@ describe Rack::Logger do
     log.info("Program started")
     log.warn("Nothing to do!")
 
-    [200, { 'Content-Type' => 'text/plain' }, ["Hello, World!"]]
+    [200, { 'content-type' => 'text/plain' }, ["Hello, World!"]]
   }
 
   it "conform to Rack::Lint" do
