@@ -2,6 +2,10 @@
 
 require_relative 'helper'
 
+separate_testing do
+  require_relative '../lib/rack/rewindable_input'
+end
+
 module RewindableTest
   extend Minitest::Spec::DSL
 
@@ -9,10 +13,6 @@ module RewindableTest
     @rio = Rack::RewindableInput.new(@io)
   end
 
-  class << self # HACK to get this running w/ as few changes as possible
-    alias_method :should, :it
-  end
-
   it "be able to handle to read()" do
     @rio.read.must_equal "hello world"
   end
@@ -40,13 +40,23 @@ module RewindableTest
   end
 
   it "rewind to the beginning when #rewind is called" do
-    @rio.read(1)
+    @rio.rewind
+    @rio.read(1).must_equal 'h'
     @rio.rewind
     @rio.read.must_equal "hello world"
   end
 
   it "be able to handle gets" do
     @rio.gets.must_equal "hello world"
+    @rio.rewind
+    @rio.gets.must_equal "hello world"
+  end
+
+  it "be able to handle size" do
+    @rio.size.must_equal "hello world".size
+    @rio.size.must_equal "hello world".size
+    @rio.rewind
+    @rio.gets.must_equal "hello world"
   end
 
   it "be able to handle each" do
@@ -55,6 +65,13 @@ module RewindableTest
       array << data
     end
     array.must_equal ["hello world"]
+
+    @rio.rewind
+    array = []
+    @rio.each do |data|
+      array << data
+    end
+    array.must_equal ["hello world"]
   end
 
   it "not buffer into a Tempfile if no data has been read yet" do
@@ -147,3 +164,12 @@ describe Rack::RewindableInput do
     include RewindableTest
   end
 end
+
+describe Rack::RewindableInput::Middleware do
+  it "wraps rack.input in RewindableInput" do
+    app = proc{|env| [200, {}, [env['rack.input'].class.to_s]]}
+    app.call('rack.input'=>StringIO.new(''))[2].must_equal ['StringIO']
+    app = Rack::RewindableInput::Middleware.new(app)
+    app.call('rack.input'=>StringIO.new(''))[2].must_equal ['Rack::RewindableInput']
+  end
+end
